@@ -7,7 +7,7 @@ proc gen_log:init {} {
   global cvsglb
 
   toplevel .trace
-  wm protocol .trace WM_DELETE_WINDOW gen_log:quit
+  wm protocol .trace WM_DELETE_WINDOW { .trace.close invoke }
   if {[info exists cvscfg(tracgeom)]} {
     wm geometry .trace $cvscfg(tracgeom)
   }
@@ -26,7 +26,8 @@ proc gen_log:init {} {
   bind .trace.bottom.entry <Return> \
       "search_textwidget .trace.text"
 
-  button .trace.bottom.quit -text "Stop Tracing" -command gen_log:quit
+  button .trace.close -text "Stop Tracing" \
+    -command { gen_log:quit; exit_cleanup 0 }
 
   pack .trace.bottom -side bottom -fill x
   pack .trace.scroll -side right -fill y
@@ -35,12 +36,13 @@ proc gen_log:init {} {
   pack .trace.bottom.srchbtn -side left
   pack .trace.bottom.entry -side left
   pack .trace.bottom.clear -side left -expand 1 -anchor c
-  pack .trace.bottom.quit -side right
+  pack .trace.close -in .trace.bottom -side right
 
   # Lets hard-code these colors - otherwise at least one is very
   # likely to blend into the bakcground
   .trace.text configure -background gray92
   .trace.text tag configure tagC -foreground purple
+  .trace.text tag configure tagE -foreground maroon
   .trace.text tag configure tagF -foreground darkgreen
   .trace.text tag configure tagT -foreground black
   .trace.text tag configure tagD -foreground red
