@@ -2,9 +2,9 @@
 
 open(HELP, "help.tcl") || die("Can't open helpl.tcl");
   
-#print ".\\\" \$Id: mkmanpage.pl,v 1.13 2005/07/19 18:16:22 dorothyr Exp $\n";
+#print ".\\\" \$Id: mkmanpage.pl,v 1.9 2004/01/02 01:38:32 dorothyr Exp $\n";
 #print ".\\\"\n";
-print ".TH TkCVS 1 Release 7.2.4\n";
+print ".TH TkCVS 1 Release 7.2\n";
 print ".SH NAME\n";
 print "TkCVS \- a Tk/Tcl Graphical Interface to CVS\n";
 print ".SH SYNOPSIS\n";
