@@ -1,5 +1,5 @@
 #
-# $Id: modbrowse.tcl,v 1.27 2002/01/23 06:55:26 dorothyr Exp $
+# $Id: modbrowse.tcl,v 1.49 2003/11/05 06:37:28 dorothyr Exp $
 #
 # Set up a check out dialog.
 #
@@ -7,21 +7,20 @@
 proc modbrowse_setup {} {
   global cwd
   global modbrowse_module
-  global mod_tagA
   global cvsglb
   global cvscfg
-  global feedback
 
   gen_log:log T "ENTER"
   set cwd [pwd]
-  set mod_tagA "HEAD"
 
   # Window manager stuff.
   toplevel .modbrowse
   wm title .modbrowse "Module Browser"
   wm iconname .modbrowse "CVS Modules"
   wm iconbitmap .modbrowse @$cvscfg(bitmapdir)/tkcvs48.xbm
-  wm protocol .modbrowse WM_DELETE_WINDOW {module_exit}
+  wm protocol .modbrowse WM_DELETE_WINDOW {
+    .modbrowse.bottom.buttons.closefm.close invoke 
+  }
   if {[info exists cvscfg(modgeom)]} {
     wm geometry .modbrowse $cvscfg(modgeom)
   } else {
@@ -44,14 +43,8 @@ proc modbrowse_setup {} {
   entry .modbrowse.top.tmcode -textvariable modbrowse_module
 
   label .modbrowse.top.lroot -text "CVSROOT"
-  entry .modbrowse.top.troot -textvariable cvscfg(cvsroot)
-  bind .modbrowse.top.troot <KeyPress-Return> {
-     ModTree:delitem .modbrowse.treeframe.pw /
-     destroy .modbrowse.treeframe.pw
-     ModTree:create .modbrowse.treeframe.pw
-     pack .modbrowse.treeframe.pw -side bottom -fill both -expand yes
-     modbrowse_run $cvscfg(cvsroot)
-  }
+  ::picklist::entry .modbrowse.top.troot cvscfg(cvsroot) cvsroot
+  ::picklist::bind .modbrowse.top.troot <KeyPress-Return> modbrowse_run
 
   button .modbrowse.top.bworkdir -image Workdir -command {workdir_setup}
 
@@ -59,25 +52,15 @@ proc modbrowse_setup {} {
   entry .modbrowse.top.tcwd -textvariable cwd
   bind .modbrowse.top.tcwd <Return> {module_changedir $cwd}
 
-  label .modbrowse.top.ltaga -text "Tag A"
-  entry .modbrowse.top.ttaga -textvariable mod_tagA
-
-  label .modbrowse.top.ltagb -text "Tag B"
-  entry .modbrowse.top.ttagb -textvariable mod_tagB
-
   grid columnconf .modbrowse.top 1 -weight 1
-  grid rowconf .modbrowse.top 4 -weight 1
-  grid .modbrowse.top.lmcode -column 0 -row 0 -sticky w
-  grid .modbrowse.top.tmcode -column 1 -row 0 -columnspan 2 -padx 4 -sticky ew
-  grid .modbrowse.top.lroot -column 0 -row 1 -sticky w
-  grid .modbrowse.top.troot -column 1 -row 1 -columnspan 2 -padx 4 -sticky ew
+  grid rowconf .modbrowse.top 3 -weight 1
+  grid .modbrowse.top.lroot -column 0 -row 0 -sticky w
+  grid .modbrowse.top.troot -column 1 -row 0 -columnspan 2 -padx 4 -sticky new
+  grid .modbrowse.top.lmcode -column 0 -row 1 -sticky w
+  grid .modbrowse.top.tmcode -column 1 -row 1 -padx 4 -sticky new
   grid .modbrowse.top.lcwd -column 0 -row 2 -sticky w
-  grid .modbrowse.top.tcwd -column 1 -row 2 -columnspan 2 -padx 4 -sticky ew
-  grid .modbrowse.top.bworkdir -column 2 -row 3 -rowspan 2 -sticky w
-  grid .modbrowse.top.ltaga -column 0 -row 3 -sticky nw
-  grid .modbrowse.top.ttaga -column 1 -row 3 -padx 4 -sticky new
-  grid .modbrowse.top.ltagb -column 0 -row 4 -sticky nw
-  grid .modbrowse.top.ttagb -column 1 -row 4 -padx 4 -sticky new
+  grid .modbrowse.top.tcwd -column 1 -row 2 -padx 4 -sticky new
+  grid .modbrowse.top.bworkdir -column 2 -row 1 -rowspan 2 -sticky w
 
   # Pack the bottom before the middle so it doesnt disappear if
   # the window is resized smaller
@@ -85,118 +68,115 @@ proc modbrowse_setup {} {
   frame .modbrowse.bottom.buttons
   frame .modbrowse.bottom.buttons.cvsfuncs -relief groove -bd 2
   frame .modbrowse.bottom.buttons.modfuncs -relief groove -bd 2
-  frame .modbrowse.bottom.buttons.close -relief groove -bd 2
+  frame .modbrowse.bottom.buttons.closefm -relief groove -bd 2
 
   pack .modbrowse.bottom -side bottom -fill x
   pack .modbrowse.bottom.buttons -side top -fill x -expand yes
-  pack .modbrowse.bottom.buttons.cvsfuncs -side left -fill x -expand yes
-  pack .modbrowse.bottom.buttons.modfuncs -side left -fill x -expand yes
-  pack .modbrowse.bottom.buttons.close -side left -fill both -expand yes
+  pack .modbrowse.bottom.buttons.closefm -side right -padx 10
+  pack .modbrowse.bottom.buttons.cvsfuncs -side left
+  pack .modbrowse.bottom.buttons.modfuncs -side left -expand yes
 
   #
   # Create buttons
   #
-  button .modbrowse.filebrowse -image Files \
+  button .modbrowse.bottom.buttons.modfuncs.filebrowse -image Files \
     -command { browse_files $modbrowse_module }
-  button .modbrowse.patchsummary -image Patches \
+  button .modbrowse.bottom.buttons.modfuncs.patchsummary -image Patches \
     -command {
-       cvs_patch_summary $modbrowse_module \
-                         $mod_tagA \
-                         $mod_tagB
+       ::dialog::patch $cvscfg(cvsroot) $modbrowse_module 1
     }
-  button .modbrowse.patchfile -image Patchfile \
+  button .modbrowse.bottom.buttons.modfuncs.patchfile -image Patchfile \
     -command {
-       cvs_patch $modbrowse_module $mod_tagA $mod_tagB
+       ::dialog::patch $cvscfg(cvsroot) $modbrowse_module 0
     }
-  button .modbrowse.checkout -image Checkout \
+  button .modbrowse.bottom.buttons.modfuncs.checkout -image Checkout \
     -command { 
-       cvsroot_check
-       cvs_checkout $modbrowse_module $mod_tagA 
+       foreach {incvs inrcs module_dir} [cvsroot_check [pwd]] { break }
+       ::dialog::checkout $cvscfg(cvsroot) $modbrowse_module
     }
-  button .modbrowse.export -image Export \
+  button .modbrowse.bottom.buttons.modfuncs.export -image Export \
     -command {
-       cvsroot_check
-       cvs_export $modbrowse_module $mod_tagA
+       foreach {incvs inrcs module_dir} [cvsroot_check [pwd]] { break }
+       ::dialog::export $cvscfg(cvsroot) $modbrowse_module
     }
-  button .modbrowse.tag -image Tag \
+  button .modbrowse.bottom.buttons.modfuncs.tag -image Tag \
     -command {
-       rtag_dialog $modbrowse_module "no"
+       rtag_dialog $cvscfg(cvsroot) $modbrowse_module "no"
     }
-  button .modbrowse.branchtag -image Branchtag \
+  button .modbrowse.bottom.buttons.modfuncs.branchtag -image Branchtag \
     -command {
-       rtag_dialog $modbrowse_module "yes"
+       rtag_dialog $cvscfg(cvsroot) $modbrowse_module "yes"
     }
-  button .modbrowse.import -image Import \
-     -command import_run
-  button .modbrowse.who -image Who \
+  button .modbrowse.bottom.buttons.cvsfuncs.import -image Import \
+     -command { import_run }
+  button .modbrowse.bottom.buttons.cvsfuncs.who -image Who \
      -command {cvs_history all $modbrowse_module}
 
-  button .modbrowse.close -text "Close" \
-    -padx 0 -pady 0 \
-    -command {module_exit}
+  button .modbrowse.bottom.buttons.closefm.close -text "Close" \
+    -command { module_exit; exit_cleanup 0 }
 
-  pack .modbrowse.who \
-       .modbrowse.filebrowse \
-       .modbrowse.checkout  \
-       .modbrowse.export  \
-    -in .modbrowse.bottom.buttons.cvsfuncs \
-    -side left -ipadx 1 -ipady 1 -fill x -expand 1
-  pack .modbrowse.tag  \
-       .modbrowse.branchtag  \
-       .modbrowse.patchsummary \
-       .modbrowse.patchfile \
-       .modbrowse.import \
-    -in .modbrowse.bottom.buttons.modfuncs \
-    -side left -ipadx 1 -ipady 1 -fill x -expand 1
-  pack .modbrowse.close \
-    -in .modbrowse.bottom.buttons.close \
-    -side right -ipadx 0 -ipady 0 -fill both -expand 1
+  grid columnconf .modbrowse.bottom.buttons.cvsfuncs 1 -weight 1
+  grid rowconf .modbrowse.bottom.buttons.cvsfuncs 0 -weight 1
+  grid .modbrowse.bottom.buttons.cvsfuncs.who -column 0 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.cvsfuncs.import -column 2 -row 0 \
+     -ipadx 4 -ipady 4
 
-  set_tooltips .modbrowse.checkout \
+  grid columnconf .modbrowse.bottom.buttons.modfuncs 7 -weight 1
+  grid rowconf .modbrowse.bottom.buttons.modfuncs 1 -weight 1
+
+  grid .modbrowse.bottom.buttons.modfuncs.filebrowse -column 1 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.modfuncs.checkout -column 2 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.modfuncs.export -column 3 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.modfuncs.tag -column 4 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.modfuncs.branchtag -column 5 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.modfuncs.patchsummary -column 6 -row 0 \
+     -ipadx 4 -ipady 4
+  grid .modbrowse.bottom.buttons.modfuncs.patchfile -column 7 -row 0 \
+     -ipadx 4 -ipady 4
+
+  pack .modbrowse.bottom.buttons.closefm.close \
+     -side right -fill both -expand yes
+
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.checkout \
      {"Check out a module from the repository"}
-  set_tooltips .modbrowse.export \
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.export \
      {"Export a module from the repository"}
-  set_tooltips .modbrowse.tag \
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.tag \
      {"Tag all files in a module"}
-  set_tooltips .modbrowse.branchtag \
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.branchtag \
      {"Branch all files in a module"}
-  set_tooltips .modbrowse.filebrowse \
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.filebrowse \
      {"Browse the files in a module"}
-  set_tooltips .modbrowse.patchsummary \
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.patchsummary \
      {"Show a summary of differences between versions"}
-  set_tooltips .modbrowse.patchfile \
+  set_tooltips .modbrowse.bottom.buttons.modfuncs.patchfile \
      {"Create a patch file"}
-  set_tooltips .modbrowse.import \
+  set_tooltips .modbrowse.bottom.buttons.cvsfuncs.import \
      {"Import the current directory into the repository"}
-  set_tooltips .modbrowse.who \
+  set_tooltips .modbrowse.bottom.buttons.cvsfuncs.who \
      {"Show who has modules checked out"}
-  set_tooltips .modbrowse.close \
+  set_tooltips .modbrowse.bottom.buttons.closefm.close \
      {"Close the module browser"}
 
   set_tooltips .modbrowse.top.bworkdir \
     {"Open the Working Directory Browser"}
 
-  set feedback(mod) [entry .modbrowse.bottom.feedback -width 55 -state disabled]
-  pack .modbrowse.bottom.feedback -side bottom -fill x -expand yes
-
   frame .modbrowse.treeframe
   pack .modbrowse.treeframe -side bottom -fill both -expand yes
   ModTree:create .modbrowse.treeframe.pw
   pack .modbrowse.treeframe.pw -side bottom -fill both -expand yes
 
-  gen_log:log T " cvsroot: $cvscfg(cvsroot)"
-  if {[string match "*:*" $cvscfg(cvsroot)]} {
-    set cvsglb(remote) 1
-  } else {
-    set cvsglb(remote) 0
-  }
-  gen_log:log T " remote=$cvsglb(remote)"
-
   set screenWidth [winfo vrootwidth .]
   set screenHeight [winfo vrootheight .]
 
   wm maxsize .modbrowse $screenWidth $screenHeight
-  wm minsize .modbrowse 10 10
+  wm minsize .modbrowse 430 300
 
   gen_log:log T "LEAVE"
 }
@@ -204,13 +184,21 @@ proc modbrowse_setup {} {
 proc modbrowse_images {} {
   global cvscfg
 
-  image create photo Files -format gif -file [file join $cvscfg(bitmapdir) files.gif]
-  image create photo Patches -format gif -file [file join $cvscfg(bitmapdir) patches.gif]
-  image create photo Patchfile -format gif -file [file join $cvscfg(bitmapdir) patchfile.gif]
-  image create photo Tag -format gif -file [file join $cvscfg(bitmapdir) tag.gif]
-  image create photo Import -format gif -file [file join $cvscfg(bitmapdir) import.gif]
-  image create photo Branchtag -format gif -file [file join $cvscfg(bitmapdir) branchtag.gif]
-  image create photo Who -format gif -file [file join $cvscfg(bitmapdir) who.gif]
+  image create photo Workdir \
+    -format gif -file [file join $cvscfg(bitmapdir) folderopen.gif]
+  image create photo Files \
+    -format gif -file [file join $cvscfg(bitmapdir) files.gif]
+  image create photo Patches \
+    -format gif -file [file join $cvscfg(bitmapdir) rdiff.gif]
+  image create photo Patchfile \
+    -format gif -file [file join $cvscfg(bitmapdir) patchfile.gif]
+  image create photo Import \
+    -format gif -file [file join $cvscfg(bitmapdir) import.gif]
+  image create photo Who \
+    -format gif -file [file join $cvscfg(bitmapdir) who.gif]
+  if {[catch "image type arr_dn"]} {
+    workdir_images
+  }
 }
 
 proc modbrowse_menus {} {
@@ -220,50 +208,41 @@ proc modbrowse_menus {} {
 
   gen_log:log T "ENTER"
 
-  menu .modmenu
-  .modbrowse configure -menu .modmenu
+  menu .modbrowse.modmenu
+  .modbrowse configure -menu .modbrowse.modmenu
 
   #
   # Create the Menu bar
   #
-  .modmenu add cascade -menu .modmenu.file -label "File" -underline 0
-  menu .modmenu.file
-  .modmenu add cascade -menu .modmenu.reports -label "Reports" -underline 0
-  menu .modmenu.reports
-  .modmenu add cascade -menu .modmenu.checkouts -label "Checkouts" -underline 0
-  menu .modmenu.checkouts
-  .modmenu add cascade -menu .modmenu.options -label "Options" -underline 0
-  menu .modmenu.options
-  .modmenu add cascade -menu .modmenu.patch -label "Patch" -underline 0
-  menu .modmenu.patch
-  .modmenu add cascade -menu .modmenu.tag -label "Tag" -underline 0
-  menu .modmenu.tag
-  .modmenu add cascade -menu .modmenu.help -label "Help" -underline 0
-  menu .modmenu.help
+  .modbrowse.modmenu add cascade -menu .modbrowse.modmenu.file -label "File" -underline 0
+  menu .modbrowse.modmenu.file
+  .modbrowse.modmenu add cascade -menu .modbrowse.modmenu.checkouts -label "Checkouts" -underline 0
+  menu .modbrowse.modmenu.checkouts
+  .modbrowse.modmenu add cascade -menu .modbrowse.modmenu.options -label "Options" -underline 0
+  menu .modbrowse.modmenu.options
+  .modbrowse.modmenu add cascade -menu .modbrowse.modmenu.patch -label "Patch" -underline 0
+  menu .modbrowse.modmenu.patch
+  .modbrowse.modmenu add cascade -menu .modbrowse.modmenu.tag -label "Tag" -underline 0
+  menu .modbrowse.modmenu.tag
 
   #
   # Create the menus
   #
   set selcolor [option get .modbrowse selectColor selectColor]
-  .modmenu.file add command -label "Browse Working Directory" -underline 0 \
+  .modbrowse.modmenu.file add command -label "Browse Working Directory" -underline 0 \
      -command workdir_setup
-  .modmenu.file add command -label "Module File" \
+  .modbrowse.modmenu.file add command -label "Module File" \
      -command module_file
-  .modmenu.file add separator
-  .modmenu.file add command -label "Import" -underline 0 \
-     -command import_run
-  .modmenu.file add command -label "Vendor Merge" -underline 0 \
+  .modbrowse.modmenu.file add separator
+  .modbrowse.modmenu.file add command -label "Import" -underline 0 \
+     -command { import_run }
+  .modbrowse.modmenu.file add command -label "Vendor Merge" -underline 0 \
      -command {merge_run $modbrowse_module}
-  .modmenu.file add separator
-  .modmenu.file add command -label "Exit" -underline 1 \
-     -command { module_exit; exit_cleanup }
+  .modbrowse.modmenu.file add separator
+  .modbrowse.modmenu.file add command -label "Exit" -underline 1 \
+     -command { module_exit; exit_cleanup 1 }
 
-  .modmenu.reports add command -label "Module Tree" -underline 0 \
-     -command {modlist_by_code $modbrowse_module 0 $mod_tagA}
-  .modmenu.reports add command -label "Version Tree" -underline 0 \
-     -command {modlist_by_code $modbrowse_module 1 $mod_tagA}
-
-  .modmenu.options add checkbutton -label "Group Aliases in a Folder" \
+  .modbrowse.modmenu.options add checkbutton -label "Group Aliases in a Folder" \
      -variable cvscfg(aliasfolder) -onvalue true -offvalue false \
      -selectcolor $selcolor -command {
         ModTree:delitem .modbrowse.treeframe.pw /
@@ -272,95 +251,100 @@ proc modbrowse_menus {} {
         pack .modbrowse.treeframe.pw -side bottom -fill both -expand yes
         modbrowse_tree [lsort [array names modval]] "/"
      }
-  .modmenu.options add separator
-  .modmenu.options add checkbutton -label "Tracing On/Off" \
+  .modbrowse.modmenu.options add separator
+  .modbrowse.modmenu.options add checkbutton -label "Tracing On/Off" \
      -variable cvscfg(logging) -onvalue true -offvalue false \
      -selectcolor $selcolor -command log_toggle
-  .modmenu.options add cascade -label "Trace Level" \
-     -menu .modmenu.options.loglevel
-  menu .modmenu.options.loglevel
-  .modmenu.options.loglevel add checkbutton -label "CVS commands (C)" \
+  .modbrowse.modmenu.options add cascade -label "Trace Level" \
+     -menu .modbrowse.modmenu.options.loglevel
+  menu .modbrowse.modmenu.options.loglevel
+  .modbrowse.modmenu.options.loglevel add checkbutton -label "CVS commands (C)" \
      -variable logclass(C) -onvalue "C" -offvalue "" \
      -selectcolor $selcolor -command gen_log:changeclass
-  .modmenu.options.loglevel add checkbutton -label "File creation/deletion (F)"\
-     -variable logclass(F) -onvalue F -offvalue "" \
+  .modbrowse.modmenu.options.loglevel add checkbutton -label "CVS stderr (E)" \
+     -variable logclass(E) -onvalue "E" -offvalue "" \
      -selectcolor $selcolor -command gen_log:changeclass
-  .modmenu.options.loglevel add checkbutton -label "Function entry/exit (T)" \
-     -variable logclass(T) -onvalue T -offvalue "" \
+  .modbrowse.modmenu.options.loglevel add checkbutton -label "File creation/deletion (F)"\
+     -variable logclass(F) -onvalue "F" -offvalue "" \
      -selectcolor $selcolor -command gen_log:changeclass
-  .modmenu.options.loglevel add checkbutton -label "Debugging (D)" \
-     -variable logclass(D) -onvalue D -offvalue "" \
+  .modbrowse.modmenu.options.loglevel add checkbutton -label "Function entry/exit (T)" \
+     -variable logclass(T) -onvalue "T" -offvalue "" \
+     -selectcolor $selcolor -command gen_log:changeclass
+  .modbrowse.modmenu.options.loglevel add checkbutton -label "Debugging (D)" \
+     -variable logclass(D) -onvalue "D" -offvalue "" \
      -selectcolor $selcolor -command gen_log:changeclass
 
-  .modmenu.checkouts add command -label "My Checkouts" -underline 0 \
+  .modbrowse.modmenu.checkouts add command -label "My Checkouts" -underline 0 \
      -command {cvs_history me ""}
-  .modmenu.checkouts add command -label "Checkouts of Selected Module" -underline 0 \
+  .modbrowse.modmenu.checkouts add command -label "Checkouts of Selected Module" -underline 0 \
      -command {cvs_history all $modbrowse_module}
-  .modmenu.checkouts add command -label "All Checkouts" -underline 0 \
+  .modbrowse.modmenu.checkouts add command -label "All Checkouts" -underline 0 \
      -command {cvs_history all ""}
 
-  .modmenu.patch add command -label "Make Patch File" -underline 0 \
+  .modbrowse.modmenu.patch add command -label "Make Patch File" -underline 0 \
      -command {
-      cvs_patch $modbrowse_module $mod_tagA $mod_tagB
+      ::dialog::patch $cvscfg(cvsroot) $modbrowse_module 0
     }
-  .modmenu.patch add command -label "View Patch Summary" -underline 0 \
+  .modbrowse.modmenu.patch add command -label "View Patch Summary" -underline 0 \
      -command {
-      cvs_patch_summary $modbrowse_module $mod_tagA $mod_tagB
+      ::dialog::patch $cvscfg(cvsroot) $modbrowse_module 1
     }
 
-  .modmenu.tag add command -label "Tag Module" -underline 0 \
+  .modbrowse.modmenu.tag add command -label "Tag Module" -underline 0 \
      -command {
-        rtag_dialog $modbrowse_module "no"
+        rtag_dialog $cvscfg(cvsroot) $modbrowse_module "no"
     }
-  .modmenu.tag add command -label "Branch Tag Module" -underline 0 \
+  .modbrowse.modmenu.tag add command -label "Branch Tag Module" -underline 0 \
      -command {
-      rtag_dialog $modbrowse_module "yes"
+      rtag_dialog $cvscfg(cvsroot) $modbrowse_module "yes"
     }
 
-  .modmenu.help add command -label "Module Browser Overview" \
-     -command module_browser
-  .modmenu.help add separator
-  .modmenu.help add command -label "Checking out Modules" \
-     -command checking_out_modules
-  .modmenu.help add command -label "Exporting" \
-     -command exporting
-  .modmenu.help add command -label "Tagging and Branching" \
-     -command tagging_and_branching
-  .modmenu.help add separator
-  .modmenu.help add command -label "Menu" \
-     -command module_browser_menu
-  .modmenu.help add command -label "Buttons" \
-     -command module_browser_buttons
-  .modmenu.help add separator
-  .modmenu.help add command -label "File Browser" \
-     -command file_browser
-  .modmenu.help add command -label "Log Browser" \
-     -command log_browser
-  .modmenu.help add separator
-  .modmenu.help add command -label "CVS modules File" \
-     -command cvs_modules_file
+  menu_std_help .modbrowse.modmenu
 
   gen_log:log T "LEAVE"
 }
 
-proc modbrowse_run {root} {
+proc modbrowse_run {} {
   global modval
   global cvscfg
+  global cvs
+  global cmd
 
-  gen_log:log T "ENTER ($root)"
+  if { $cvscfg(cvsroot) == "" } {
+    cvsfail "I don't know what \$CVSROOT is!"
+    return
+  }
+
+  if {[info exists cmd(cvs_co)]} {
+    catch {$cmd(cvs_co)\::abort}
+    unset cmd(cvs_co)
+  }
+
+  set cmd(cvs_co) [exec::new "$cvs -d $cvscfg(cvsroot) checkout -p CVSROOT/modules"]
 
   set modbrowse_module ""
-  read_modules $root
 
   if {! [winfo exists .modbrowse]} {
     modbrowse_setup
+  } else {
+     ModTree:delitem .modbrowse.treeframe.pw /
+     destroy .modbrowse.treeframe.pw
+     ModTree:create .modbrowse.treeframe.pw
+     pack .modbrowse.treeframe.pw -side bottom -fill both -expand yes
   }
+  ::picklist::used cvsroot $cvscfg(cvsroot)
+
+  busy_start .modbrowse
+  wm deiconify .modbrowse
+  raise .modbrowse
+
+  parse_modules [$cmd(cvs_co)\::output]
+  catch {unset cmd(cvs_co)}
 
   # Populate the tree
   modbrowse_tree [lsort [array names modval]] "/"
 
-  wm deiconify .modbrowse
-  raise .modbrowse
+  busy_done .modbrowse
   gen_log:log T "LEAVE"
 }
 
@@ -505,37 +489,42 @@ proc modbrowse_select_code {yposition} {
 proc module_exit { } {
   global cvscfg
   global cvs
+  global cmd
 
   gen_log:log T "ENTER"
 
 
+  if {[info exists cmd(cvs_co)]} {
+    catch {$cmd(cvs_co)\::abort}
+    unset cmd(cvs_co)
+  }
+
   set pid [pid]
   set cwd [pwd]
   set sandbox [file join $cvscfg(tmpdir) cvstmpdir.$pid]
   if {[file isdirectory $sandbox]} {
     gen_log:log F "CD $sandbox"
     cd $sandbox
+    set dirs {}
     foreach d [glob -nocomplain *] {
-      gen_log:log C "$cvs -Q release $d"
-      catch {eval exec $cvs -Q release $d}
+      lappend dirs $d
     }
+    gen_log:log C "$cvs -Q release $dirs"
+    catch {eval "exec $cvs -Q release $dirs"}
+    # Doing it this way makes it pop up an error on windoze.
+    # Very annoying.
+    #set finish [exec::new "$cvs -Q release $dirs"]
+    #$finish\::wait
   }
   cd $cwd
   gen_log:log F "cd [pwd]"
 
-  if {[winfo exists .modbrowse]} {
-    ModTree:delitem .modbrowse.treeframe.pw /
-    set cvscfg(modgeom) [wm geometry .modbrowse]
-    destroy .modbrowse
-    destroy .modmenu
-    catch {destroy .tooltips_wind}
-  }
+  ModTree:delitem .modbrowse.treeframe.pw /
+  set cvscfg(modgeom) [wm geometry .modbrowse]
+  destroy .modbrowse
+  catch {destroy .tooltips_wind}
+  exit_cleanup 0
 
-  # If no workdir browser, quit entirely.  Otherwise, just quit the
-  # module browser.
-  if {! [winfo exists .workdir]} {
-    exit_cleanup
-  }
   gen_log:log T "LEAVE"
 }
 
@@ -554,11 +543,11 @@ proc module_changedir {new_dir} {
     if {[file isdirectory $cvsdir]} {
       read_cvs_dir $cvsdir
       if {$savroot != $cvscfg(cvsroot)} {
-        read_modules $cvscfg(cvsroot)
+        modbrowse_run
       }
     }
     if {[winfo exists .workdir]} {
-      update_go $new_dir
+      ::picklist::used directory [pwd]
       setup_dir
     }
   } else {
@@ -573,6 +562,8 @@ proc module_file { } {
   global cvscfg
 
   set commandline "$cvs -d $cvscfg(cvsroot) checkout -p CVSROOT/modules"
-  gen_log:log C "$commandline"
-  exec_command "CVSROOT/modules" "$commandline"
+  #gen_log:log C "$commandline"
+  set v [viewer::new "CVSROOT/modules"]
+  $v\::do "$commandline"
 }
+
