@@ -1,5 +1,5 @@
 #
-# $Id: tkcvs_def.tcl,v 1.35 2002/01/21 06:57:49 dorothyr Exp $
+# $Id: tkcvs_def.tcl,v 1.47 2004/01/02 06:40:26 dorothyr Exp $
 #
 # TkCVS defaults file.
 #
@@ -10,13 +10,6 @@
 # over-ride this file.
 #
 
-# Print setup.
-set cvscfg(papersize) "A4"
-set cvscfg(pointsize) 10
-set cvscfg(headingsize) 13
-set cvscfg(subheadingsize) 11
-set cvscfg(printer) "ps"
-
 # If you want to use "cvs edit", uncomment this.
 set cvscfg(econtrol) true
 # If you want to see the status column, uncomment this
@@ -75,7 +68,7 @@ if { [winfo depth .] == 1 } {
 #
 #   %Y/%m/%d %H:%M:%S    - 2000/03/25 14:41:33
 #
-# is useful because it sorts properly. Other poosibilities
+# is useful because it sorts properly. Other possibilities
 # are:
 #
 #   %d/%m/%y %I:%M:%S %p   - 03/25/00 02:41:33 PM
@@ -86,6 +79,100 @@ if { [winfo depth .] == 1 } {
 #
 #set cvscfg(dateformat) "%Y/%m/%d %H:%M:%S"
 set cvscfg(dateformat) "%Y/%m/%d %H:%M"
+# Format for mergeto- and mergefrom- tags.  The _BRANCH_ part must be
+# left as-is, but you can change the prefix and the date format, for
+# example "mergeto_BRANCH_%d%b%y".  The date format must be the same
+# for both.
+# CVS rule: a tag must not contain the characters `$,.:;@'
+#set cvscfg(mergetoformat) "t_BRANCH_%d%b%y"
+#set cvscfg(mergefromformat) "f_BRANCH_%d%b%y"
+set cvscfg(mergetoformat) "mergeto_BRANCH_%d%b%y"
+set cvscfg(mergefromformat) "mergefrom_BRANCH_%d%b%y"
+
+# --------------------
+# Revision tree log display configuration.
+
+# Font size for tag lists and box contents (+ve = points, -ve = pixels)
+set logcfg(font_size) -12
+
+# Gaps between revisions in units of the chosen font's line spacing
+# spcx = x spacing between revisions
+# spcy = y spacing between revisions
+# yfudge = max extra y space used to fit branch in rather than moving right
+# boff = vertical offset for branch placement
+set logcfg(spcx) 1
+set logcfg(spcy) 1
+set logcfg(yfudge) 12
+set logcfg(boff) 1
+
+# Padding between box outline and box contents in pixels
+set logcfg(padx) 4
+set logcfg(pady) 2
+
+# Space between tag list and box in pixels
+set logcfg(tspcb) 2
+
+# Line and box outline width in pixels
+set logcfg(width) 3
+
+# Arrow shape for connecting lines
+set logcfg(arrowshape) { 6 6.7 3 }
+
+# Delay between a user option being changed and the redraw of the
+# tree taking place. This is to allow the user chance to change
+# several options at once without the tree being redrawn unecessarily.
+# It's in milliseconds and something in the 1.5-3 second range is
+# generally reasonable.
+set logcfg(draw_delay) 2000
+
+# Scaling options to offer user
+set logcfg(scaling_options) {50% 0.5 75% 0.75 100% 1.0 150% 1.5 200% 2.0 250% 2.5}
+
+# User options for info display
+set logcfg(update_drawing) 2
+set logcfg(scale) 1.0
+set logcfg(show_tags) 1
+set logcfg(show_merges) 1
+set logcfg(show_empty_branches) 1
+set logcfg(show_inter_revs) 1
+set logcfg(show_root_tags) 1
+set logcfg(show_root_rev) 1
+set logcfg(show_box_rev) 1
+set logcfg(show_box_revwho) 1
+set logcfg(show_box_revdate) 1
+set logcfg(show_box_revtime) 0
+set logcfg(show_box_revlines) 0
+set logcfg(show_box_revstate) 0
+
+
+
+# --------------------
+# Import defaults
+set cvscfg(import_vendortag) {VENDOR}
+set cvscfg(import_vendorbranch) {1.1.1}
+# Standalone CVS import behaviour
+#set cvscfg(import_newtrunk) 2
+#set cvscfg(import_missdel) 0
+# Recommended settings
+set cvscfg(import_newtrunk) 0
+set cvscfg(import_missdel) 1
+set cvscfg(import_alttime) {}
+# It is *highly* recommended that you configure cvswrappers to set -kb for
+# binary file types and -ko for all others, then change the default kflag
+# below to {} rather than {-ko}.
+set cvscfg(import_kflag) {-ko}
+# You should clean your import directory by hand unless you are absolutely
+# *certain* you know what ignore patterns are in effect - otherwise you may
+# lose files that never get imported!
+set cvscfg(import_ignore) {!}
+
+# If you have standard files that you want present in a module when it is
+# created by import without a directory specified (e.g. README, COPYRIGHT
+# etc.) then you can put them in a directory and set cvscfg(module_template)
+# to its name. Note that importing to an existing module without a directory
+# specified will also add the template files to both the vendor branch and
+# the trunk.
+#set cvscfg(module_template) {}
 
 
 # --------------------
@@ -101,6 +188,8 @@ if {$tcl_platform(platform) == "windows"} {
     set cvscfg(aster) "*.*"
     # null-device
     set cvscfg(null) "nul"
+    # Terminal program
+    set cvscfg(terminal) "command /c"
     # Please don't ask me why you have to set -T on DOS,
     # experiments say you have! - CJ
     #set cvs "cvs -T $cvscfg(tmpdir)"
@@ -124,6 +213,8 @@ if {$tcl_platform(platform) == "windows"} {
     set cvscfg(tmpdir) "/tmp"
     set cvscfg(aster) "*"
     set cvscfg(null) "/dev/null"
+    # Terminal program
+    set cvscfg(terminal) "xterm -e"
     #
     # Other defaults
     #
@@ -160,6 +251,11 @@ if {$tcl_platform(platform) == "windows"} {
     set cvscfg(allow_abort)    "yes"
     # What do you want to happen when you ask for a shell?
     set cvscfg(shell) "xterm -name tkcvsxterm -n {TkCVS xterm}"
+    if {! [catch {set windowingsystem [tk windowingsystem]}] && $windowingsystem == "aqua"} {
+      set cvscfg(editor) /Applications/TextEdit.app/Contents/MacOS/textedit
+      set cvscfg(tkdiff) "\"$ScriptBin/../../MacOS/Wish Shell\" $ScriptBin/tkdiff.tcl"
+      set cvscfg(shell) /Applications/Utilities/Terminal.app/Contents/MacOS/Terminal
+    }
 }
 
 #
@@ -200,7 +296,7 @@ set cvscfg(rdetail)            "summary"
 
 # set the default pattern to be used by the filter.  Use any valid
 # pattern that can be used for a pattern for 'ls'. An empty string
-# is equilivant to the entire directory (minus hidden files);
+# is equivalent to the entire directory (minus hidden files);
 # i.e., ls *
 set cvscfg(file_filter)        ""
 set cvscfg(ignore_file_filter) "*.a *.o *~"
@@ -218,13 +314,15 @@ set cvscfg(confirm_prompt)     true
 # some of the reporting operations could usefully be recursive.  Set
 # the default value here.
 set cvscfg(recurse)            false
+# Filter out "?" unknown files from CVS Check and CVS Update reports
+set cvscfg(status_filter)      false
 
 # Kinds of messages for debugging:
 #         C       CVS commands
 #         F       File creation/deletion
 #         T       Function entry/exit tracing
 #         D       Debugging"
-set cvscfg(log_classes) "CF"
+set cvscfg(log_classes) "CEF"
 # On (1) or off (0)
 set cvscfg(logging)    false
 
@@ -240,6 +338,24 @@ set cvscfg(aliasfolder) true
 #
 set cvscfg(dayspercolor) 20
 
+# Set colours for tagging cvs output
+set cvscfg(outputColor,patched) blue4
+set cvscfg(outputColor,modified) purple
+set cvscfg(outputColor,conflict) red
+set cvscfg(outputColor,updated) darkgoldenrod
+set cvscfg(outputColor,added)   darkgreen
+set cvscfg(outputColor,removed) maroon
+set cvscfg(outputColor,warning) orange
+set cvscfg(outputColor,unknown) gray30
+set cvscfg(outputColor,stderr) darkred
+
+# Print setup. Removed in v7.1
+#set cvscfg(papersize) "A4"
+#set cvscfg(pointsize) 10
+#set cvscfg(headingsize) 13
+#set cvscfg(subheadingsize) 11
+#set cvscfg(printer) "ps"
+
 #
 # --------------------
 # At the very end, look for a file called "site_def" in the installation
